#!/usr/bin/env python

import io
import pandas as pd
from pysam import VariantFile
from pyvariantfilter.variant import Variant
import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt
import scipy

sns.set()

def calculate_upd_metrics_per_chromosome(vcf, chromosome_to_analyze, family, block_size, min_dp ,min_gq, min_qual, proband_id):
	
	family_member_ids = family.get_all_family_member_ids()

	bcf_in = VariantFile(vcf)

	variant_count = 0
	matches_paternal_uniparental_ambiguous_count = 0
	matches_maternal_uniparental_ambiguous_count = 0
	matches_paternal_uniparental_isodisomy_count = 0
	matches_maternal_uniparental_isodisomy_count = 0
	alleles_identical_to_dad_count = 0
	alleles_identical_to_mum_count = 0
	is_homozygous_count = 0
	is_biparental_count = 0

	block_dict = {}
	variant_list = []

	last_block = 0
	block_count = 1

	for rec in bcf_in.fetch(chromosome_to_analyze):
		chrom = rec.chrom
		pos = rec.pos
		ref = rec.ref
		alt = rec.alts
		filter_status = rec.filter.keys()
		quality = rec.qual

		if len(alt) != 1:

			continue

		alt = alt[0]

		if alt == '*':
			continue

		if quality <= min_qual:

			continue

		new_variant = Variant(chrom=chrom.strip('chr'), pos=pos, ref=ref, alt=alt, filter_status=filter_status, quality=quality)
		new_variant.add_family(family)

		for family_member_id in family_member_ids:

			sample_genotype_data = rec.samples[family_member_id]

			ref_and_alt = [ref, alt]

			gts =[]
			ads =[]

			for allele in sample_genotype_data['GT']:

				if allele == None:

					gts.append('.')

				else:

					gts.append(ref_and_alt[allele])

			if len(gts) == 1 and (chrom == 'X' or chrom == 'Y' or chrom == 'chrX' or chrom == 'chrY'):

				gts.append(gts[0])

			if gts[0] == '.' and gts[1] == '.':

					ads.append(0)
					ads.append(0)

			elif len(sample_genotype_data['AD']) == 1 and sample_genotype_data['AD'][0] == None:

					ads.append(0)
					ads.append(0)

			else:

				assert len(sample_genotype_data['AD']) == 2

				for ad in sample_genotype_data['AD']:

					if ad == None:

						ads.append(0)

					else:

						ads.append(ad)

			gq = sample_genotype_data['GQ']

			if gq == None:

				gq = 0

			dp = sample_genotype_data['DP']

			if dp == None:

				dp  = 0
			
			af = sample_genotype_data['AF']

			if af == None:

				af  = 0

			else:

				af = af[0]

			try:
				new_variant.add_genotype(family_member_id, gts, ads, gq, dp)
			except:
				print(chrom, pos, ref, alt, gts)
				raise

		# now we have created variant object let us count the number for each block

		if new_variant.is_snp() and new_variant.all_samples_pass_genotype_quality(min_dp, min_gq):

			variant_list.append([chrom, pos, ads[0], ads[1], af, dp, gq, quality])

			paternal_uniparental_ambiguous = new_variant.matches_paternal_uniparental_ambiguous(min_parental_gq = min_gq, min_parental_depth = min_dp)
			maternal_uniparental_ambiguous = new_variant.matches_maternal_uniparental_ambiguous(min_parental_gq = min_gq, min_parental_depth = min_dp)

			paternal_uniparental_isodisomy = new_variant.matches_paternal_uniparental_isodisomy(min_parental_gq = min_gq, min_parental_depth = min_dp)
			maternal_uniparental_isodisomy = new_variant.matches_maternal_uniparental_isodisomy(min_parental_gq = min_gq, min_parental_depth = min_dp)

			alleles_identical_to_dad =  new_variant.alleles_identical_to_dad(min_parental_gq = min_gq, min_parental_depth = min_dp)
			alleles_identical_to_mum = new_variant.alleles_identical_to_mum(min_parental_gq = min_gq, min_parental_depth = min_dp)

			is_homozygous = new_variant.is_homozygous(proband_id)

			is_biparental = new_variant.is_biparental_inheritance(min_parental_gq = min_gq, min_parental_depth = min_dp)

			variant_count += 1

			if paternal_uniparental_ambiguous:

				matches_paternal_uniparental_ambiguous_count += 1

			if maternal_uniparental_ambiguous:

				matches_maternal_uniparental_ambiguous_count += 1

			if paternal_uniparental_isodisomy:

				matches_paternal_uniparental_isodisomy_count += 1

			if maternal_uniparental_isodisomy:

				matches_maternal_uniparental_isodisomy_count += 1

			if alleles_identical_to_dad:

				alleles_identical_to_dad_count += 1

			if alleles_identical_to_mum:

				alleles_identical_to_mum_count += 1

			if is_homozygous:

				is_homozygous_count += 1

			if is_biparental:

				is_biparental_count += 1

			# have we crossed a new block 
			if new_variant.pos > last_block + block_size:

				block_dict[block_count] = {
					'end': block_count * block_size,
					'variant_count' : variant_count,
					'matches_paternal_uniparental_ambiguous_count' : matches_paternal_uniparental_ambiguous_count,
					'matches_maternal_uniparental_ambiguous_count': matches_maternal_uniparental_ambiguous_count,
					'matches_paternal_uniparental_isodisomy_count': matches_paternal_uniparental_isodisomy_count,
					'matches_maternal_uniparental_isodisomy_count': matches_maternal_uniparental_isodisomy_count,
					'alleles_identical_to_dad_count': alleles_identical_to_dad_count,
					'alleles_identical_to_mum_count': alleles_identical_to_mum_count,
					'is_homozygous_count': is_homozygous_count,
					'is_biparental_count': is_biparental_count

				}

				variant_count = 0
				matches_paternal_uniparental_ambiguous_count = 0
				matches_maternal_uniparental_ambiguous_count = 0
				matches_paternal_uniparental_isodisomy_count = 0
				matches_maternal_uniparental_isodisomy_count = 0
				alleles_identical_to_dad_count = 0
				alleles_identical_to_mum_count = 0
				is_homozygous_count = 0
				is_biparental_count = 0

				last_block = last_block + block_size


				block_count = block_count +1


	return block_dict, variant_list

def replace_with_na(df, column, min_count):

	if int(df['variant_count']) < int(min_count):
		
		return np.nan
		
	else:
		
		return df[column]



def create_ax_for_plotting(chromosome, df, block_size, output):

	chrom_prop_df = df[df['chrom'] ==chromosome]

	plot_min = min(chrom_prop_df['end'])
	plot_max = max(chrom_prop_df['end'])

	# we want to scale the X axis labels so that it looks pretty
	chrom_size = plot_max

	if chrom_size > 100000000:
		
		scale_factor = 10
		
	else:
		
		scale_factor = 3

	xticks_chrom = np.arange(plot_min, plot_max, block_size*scale_factor)
	xticks_labels = np.arange(plot_min, plot_max, block_size*scale_factor)
	x_ticks_labels = [int(x/1000000) for x in xticks_chrom]

	# plot and format axis
	ax = chrom_prop_df.plot(x='end', figsize=(20,5))
	ax.set_xticks(xticks_chrom)
	ax.set_xlim([plot_min-1000000, plot_max+1000000])
	ax.set_ylim([-0.02, 1.05])
	ax.set_xticklabels(x_ticks_labels)
	ax.set_xlabel(f'Chromosome {chromosome} Position (Mb)')
	ax.set_ylabel('Proportion of Variants')
	ax.legend(loc='upper right')

	plt.savefig(output, format='png')

	plt.close('all')

def plot_variants(chromosome, df, output, block_size, sample_fraction):
	"""
	Plots allele frequency vs. position for a given chromosome with optional downsampling.

	Parameters:
	chromosome (str): The chromosome to plot.
	df (pd.DataFrame): DataFrame containing the variant data with 'chrom', 'pos', and 'af' columns.
	output (str): Path to save the plot as a PNG file.
	block_size (int): Block size for xticks calculation.
	sample_fraction (float): Fraction of data to sample for plotting (default: 0.1).
	"""
	# Filter data for the specific chromosome

	chrom_prop_df = df[df['chrom'] == chromosome]

	# Downsample the data
	if sample_fraction < 1.0:
		chrom_prop_df = chrom_prop_df.sample(frac=sample_fraction, random_state=42)

	# Calculate plot range and scaling
	plot_min = chrom_prop_df['pos'].min()
	plot_max = chrom_prop_df['pos'].max()

	chrom_size = plot_max

	scale_factor = 10 if chrom_size > 100000000 else 3

	xticks_chrom = np.arange(plot_min, plot_max, block_size * scale_factor)
	x_ticks_labels = [int(x / 1000000) for x in xticks_chrom]

	# Plot and format axis
	fig, ax = plt.subplots(figsize=(20, 5))
	ax.scatter(x='pos', y='af', data=chrom_prop_df, s=2, alpha=0.5)
	ax.set_xticks(xticks_chrom)
	ax.set_xlim([plot_min - 1000000, plot_max + 1000000])
	ax.set_ylim([-0.02, 1.05])
	ax.set_xticklabels(x_ticks_labels)
	ax.set_xlabel(f'Chromosome {chromosome} Position (Mb)')
	ax.set_ylabel('Beta Allele Frequency')

	# Save and close plot
	plt.savefig(output, format='png')
	plt.close('all')


def is_significant(df, expected, key):
	
	if df[key] == None:
		
		return None
	
	mendel_errors = df[key]
	
	variant_count = df['variant_count']
	
	
	return scipy.stats.binomtest(mendel_errors, variant_count, expected, alternative='greater').pvalue


def merge_contiguous_blocks(df, block_size, analysis, analysis2):

	if df.shape[0] == 0:

		return []

	contiguous_blocks = []

	last_chr = 'NA'
	last_start = None
	last_end = None

	current_block_start = None
	current_block_chr = None

	current_p_values = []
	current_proportions = []

	for row in df.itertuples():
		
		current_chr = row.chrom
		current_end = row.end
		current_start = current_end - block_size

		if current_chr != last_chr:
			
			if last_chr != 'NA':
				
				contiguous_blocks.append([last_chr, current_block_start, last_end, np.mean(current_p_values), np.mean(current_proportions)])
				
				current_p_values = []
				
			current_block_start = current_start
			last_chr = current_chr
			
		elif current_start != (last_start+block_size):
			
			contiguous_blocks.append([current_chr, current_block_start, last_end, np.mean(current_p_values), np.mean(current_proportions)])
			current_block_start = current_start
			current_p_values  =[]
			
		
		last_chr = current_chr
		last_start = current_start
		last_end = current_end
		current_p_values.append(row.__getattribute__(analysis))
		current_proportions.append(row.__getattribute__(analysis2))
			
	contiguous_blocks.append([last_chr, current_block_start, last_end, np.mean(current_p_values), np.mean(current_proportions)])

	return contiguous_blocks

def apply_filters(df, min_blocks, min_proportion, block_size):

	filters = []

	proportion = df['mean_proportion_me']
	start = df['start']
	end = df['end']

	if proportion < min_proportion:

		filters.append('low_proportion_me')

	block_length = end - start

	n_blocks = block_length / block_size

	if n_blocks < min_blocks:

		filters.append('min_block_length')

	if len(filters) == 0:

		return 'pass'

	else:

		return ';'.join(filters)

def get_genome_build(vcf):
	"""
	Get the genome build from the header
	"""
	bcf_in = VariantFile(vcf)
	# xticks_chrom = np.arange(plot_min, plot_max, block_size*scale_factor)
	# xticks_labels = np.arange(plot_min, plot_max, block_size*scale_factor)
	# x_ticks_labels = [int(x/1000000) for x in xticks_chrom]
	header = bcf_in.header

	header =  str(header)

	for line in io.StringIO(header).readlines():

		if '##reference' in line:

			if 'hs37d5' in line or 'human_g1k_v37' in line or 'GRCh37' in line or 'hg19' in line:

				return 37

			elif 'Homo_sapiens_assembly38' in line or 'GRCh38' or 'hg38' in line:

				return 38

	raise Exception('Invalid reference genome in VCF header')


#!/usr/bin/env python

import argparse
import csv
import sys
import pandas as pd
from pyvariantfilter.family import Family
from pyvariantfilter.family_member import FamilyMember
from pyvariantfilter.variant_set import VariantSet
from upd.utility_funcs import calculate_upd_metrics_per_chromosome, create_ax_for_plotting, replace_with_na,is_significant, merge_contiguous_blocks, apply_filters, get_genome_build, plot_variants
from pysam import VariantFile

parser = argparse.ArgumentParser(description='Find UPD events in NGS Trio Data')
parser.add_argument('--vcf', type=str, required=True,
				help='The path to the VCF file. Must be bgzipped and tabixed.')
parser.add_argument('--proband_id', type=str, required=True,
				help='The Sample ID of the proband in the VCF.')
parser.add_argument('--ped', type=str, required=True,
				help='A ped file describing the family relationships.')
parser.add_argument('--output', type=str, required=True, help='The output name prefix.')
parser.add_argument('--min_dp', type=int, required=False, default=20, help='The minimum genotype depth. Default = 20')
parser.add_argument('--block_size', type=int, required=False, default=1000000, help='The block size for calculating areas of chromsome affected by UPD. Default = 1000000')
parser.add_argument('--min_gq', type=int, required=False, default=20, help='The minimum genotype quality (GQ). Default = 20')
parser.add_argument('--min_qual', type=int,required=False, default=90, help='The minimum QUAL value. Default = 90')
parser.add_argument('--min_variants_per_block', default=100, help='The minimum number of variants in a block. Default = 100')
parser.add_argument('--p_value', type=float, required=False, default=0.001, help='The maximum P value for statistical test for block significance. Default = 0.001')
parser.add_argument('--chromosome', type=str, required=False, help='Restrict to single chromosome. WARNING: For testing purposes only. E.g. "chr22"')
parser.add_argument('--min_blocks', type=int, required=False, default=5, help='The minimum number of contiguous blocks for a call not to be filtered. Default = 5')
parser.add_argument('--min_proportion', type=float, required=False, default=0.01, help='If the proportion of UPD variants in a contiguous block is below this then apply a filter. Default = 0.01')
parser.add_argument('--prop_plot', type=bool, required=False, default=False, help='Plot proportion of variants plot (BAF) per chromosome. True/False, default = False')
parser.add_argument('--downsample', type=float, required=False, default=0.3, help='Downsample variants for BAF plot by this proportion.')

args = parser.parse_args()

vcf = args.vcf
proband_id = args.proband_id
ped = args.ped
output = args.output
min_dp = args.min_dp
min_gq = args.min_gq
min_qual = args.min_qual
p_value = args.p_value
block_size = args.block_size
min_variants_per_block = args.min_variants_per_block
min_blocks = args.min_blocks
min_proportion = args.min_proportion
prop_plot = args.prop_plot
downsample = args.downsample

if args.chromosome != None:

	chromosome = args.chromosome
	just_one_chromosome = True

else:

	just_one_chromosome = False


# check which genome build we have

genome_build = get_genome_build(vcf)

# read ped into df
ped_df = pd.read_csv(ped, sep='\t', names=['family_id', 'sample_id', 'paternal_id', 'maternal_id', 'sex', 'affected'], dtype={'sex': str,
																															 'affected': str,
																															 'paternal_id': str,
																															 'maternal_id': str,
																															 'family_id': str,
																															 })
# filter by proband
filtered_ped = ped_df[ped_df['sample_id']==proband_id]

# get mum nad dad ids
dad = filtered_ped['paternal_id'].iloc[0]
mum = filtered_ped['maternal_id'].iloc[0]
sex = filtered_ped['sex'].iloc[0]
affected = filtered_ped['affected'].iloc[0]

# get family id
family_id = filtered_ped['family_id'].iloc[0]


# Record as singleton if we don't have mum and dad
if dad == '0' or mum == '0':

	singleton = True

else:

	singleton = False


if affected != '2':

	print('Sample must be affected in PED to create plots. Set affected solumn to 2.')
	sys.exit(0)


if family_id == '0':

	proband = FamilyMember(proband_id, proband_id, int(sex), True)

	my_family = Family(proband_id)

	my_family.add_family_member(proband)

	my_family.set_proband(proband.get_id())

else:

	# make a family object
	my_family = Family(family_id)
	my_family.read_from_ped_file(ped, family_id, proband_id)

# check which chromosomes to analyse
if just_one_chromosome == True:

	if sex == '1' and chromosome == 'X':

		print ('Chromosome cannot be X if proband is male.')
		exit()

	else:

		chromosomes_to_analyze = [chromosome]

else:

	chromosomes_to_analyze = ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', 'X']

	if sex == '2':

		print ('Proband is Female - will analyse chromosomes 1-22 and X')

	elif sex == '1':

		print ('Proband is Male - will analyse chromosomes 1-22 only.')
		chromosomes_to_analyze.pop()

	elif sex == '0':

		print ('Sex not set assuming proband is Male - will analyse chromosomes 1-22 only.')
		chromosomes_to_analyze.pop()
		sex = '1'	

	if genome_build == 38:

		print('Genome build is 38')
		chromosomes_to_analyze = ['chr' + c for c in chromosomes_to_analyze]

	else:

		print ('Genome build is 37')

print ('Analysing chromosomes', ', '.join(chromosomes_to_analyze))

master_df_list = []
variants_df_list = []

for chromosome in chromosomes_to_analyze:

	print(f'Calculating UPD Metrics for Chromosome {chromosome}')
	
	block_dict, variants_list = calculate_upd_metrics_per_chromosome(vcf, chromosome, my_family, block_size, min_dp, min_gq, min_qual, proband_id)
	
	df = pd.DataFrame(block_dict).transpose()
	df['chrom'] = chromosome
	
	master_df_list.append(df)
	
	if prop_plot:
		
		df_variants = pd.DataFrame(variants_list, columns=['chrom', 'pos', 'ref_count', 'alt_count', 'af', 'dp', 'gq', 'qual'])
		variants_df_list.append(df_variants)

# Combine all collected dataframes at the end
master_df = pd.concat(master_df_list, ignore_index=True)

if prop_plot:
	variants_df = pd.concat(variants_df_list, ignore_index=True)

# convert columns to proportions e.g. proportion of variants with errors
master_df['prop_alleles_identical_to_dad_count'] = master_df['alleles_identical_to_dad_count']/master_df['variant_count']
master_df['prop_alleles_identical_to_mum_count'] = master_df['alleles_identical_to_mum_count']/master_df['variant_count']
master_df['prop_is_biparental_count'] = master_df['is_biparental_count']/master_df['variant_count']
master_df['prop_is_homozygous_count'] = master_df['is_homozygous_count']/master_df['variant_count']
master_df['prop_matches_maternal_uniparental_ambiguous_count'] = master_df['matches_maternal_uniparental_ambiguous_count']/master_df['variant_count']
master_df['prop_matches_maternal_uniparental_isodisomy_count'] = master_df['matches_maternal_uniparental_isodisomy_count']/master_df['variant_count']
master_df['prop_matches_paternal_uniparental_ambiguous_count'] = master_df['matches_paternal_uniparental_ambiguous_count']/master_df['variant_count']
master_df['prop_matches_paternal_uniparental_isodisomy_count'] = master_df['matches_paternal_uniparental_isodisomy_count']/master_df['variant_count']
master_df['prop_me'] = master_df['prop_matches_maternal_uniparental_ambiguous_count'] + master_df['prop_matches_maternal_uniparental_isodisomy_count'] + master_df['prop_matches_paternal_uniparental_ambiguous_count'] + master_df['prop_matches_paternal_uniparental_isodisomy_count']

master_df['prop_alleles_identical_to_dad_count'] = master_df.apply(replace_with_na, axis=1, args=('prop_alleles_identical_to_dad_count', min_variants_per_block,))
master_df['prop_alleles_identical_to_mum_count'] = master_df.apply(replace_with_na, axis=1, args=('prop_alleles_identical_to_mum_count', min_variants_per_block,))
master_df['prop_is_biparental_count'] = master_df.apply(replace_with_na, axis=1, args=('prop_is_biparental_count', min_variants_per_block,))
master_df['prop_is_homozygous_count'] = master_df.apply(replace_with_na, axis=1, args=('prop_is_homozygous_count', min_variants_per_block,))
master_df['prop_matches_maternal_uniparental_ambiguous_count'] = master_df.apply(replace_with_na, axis=1, args=('prop_matches_maternal_uniparental_ambiguous_count', min_variants_per_block,))
master_df['prop_matches_maternal_uniparental_isodisomy_count'] = master_df.apply(replace_with_na, axis=1, args=('prop_matches_maternal_uniparental_isodisomy_count', min_variants_per_block,))
master_df['prop_matches_paternal_uniparental_ambiguous_count'] = master_df.apply(replace_with_na, axis=1, args=('prop_matches_paternal_uniparental_ambiguous_count', min_variants_per_block,))
master_df['prop_matches_paternal_uniparental_isodisomy_count'] = master_df.apply(replace_with_na, axis=1, args=('prop_matches_paternal_uniparental_isodisomy_count', min_variants_per_block,))
master_df['prop_me'] = master_df.apply(replace_with_na, axis=1, args=('prop_me', min_variants_per_block,))


# plot data and save to file
if singleton:

	prop_df = master_df[[
		   'prop_is_homozygous_count',
		   'end',
		  'chrom']]
else:
	prop_df = master_df[[
			'prop_is_homozygous_count',
			'prop_matches_maternal_uniparental_ambiguous_count',
			'prop_matches_maternal_uniparental_isodisomy_count',
			'prop_matches_paternal_uniparental_ambiguous_count',
			'prop_matches_paternal_uniparental_isodisomy_count',
			'end',
			'chrom']]

for chromosome in chromosomes_to_analyze:

	print (f'Plotting Metrics for chromosome: {chromosome}')

	plot_location = f'{output}_{chromosome}_UPD.png'
	variant_plot_location = f'{output}_{chromosome}_baf.png'

	# create upd plot from genotypes
	create_ax_for_plotting(chromosome, prop_df, block_size, plot_location)
	
	# create allele frequency plot
	if prop_plot:
		
		print(f'Plotting {len(variants_df.index)} variants in proportion of variants plot {chromosome}')
		
		plot_variants(chromosome, variants_df, variant_plot_location, block_size, downsample)


# get mean so we know what expected ratio is i.e. that caused by errors - hmm what if every chromosome is UPD?
mean_matches_maternal_uniparental_isodisomy_count = master_df['matches_maternal_uniparental_isodisomy_count'].sum() / master_df['variant_count'].sum()
mean_matches_maternal_uniparental_ambiguous_count = master_df['matches_maternal_uniparental_ambiguous_count'].sum() / master_df['variant_count'].sum()

mean_matches_paternal_uniparental_ambiguous_count = master_df['matches_paternal_uniparental_ambiguous_count'].sum() / master_df['variant_count'].sum()
mean_matches_paternal_uniparental_isodisomy_count = master_df['matches_paternal_uniparental_isodisomy_count'].sum() / master_df['variant_count'].sum()

master_df['sig_prop_matches_maternal_uniparental_ambiguous_count'] = master_df.apply(is_significant, axis=1, args=(mean_matches_maternal_uniparental_ambiguous_count, 'matches_maternal_uniparental_ambiguous_count', ))
master_df['sig_prop_matches_maternal_uniparental_isodisomy_count'] = master_df.apply(is_significant, axis=1, args=(mean_matches_maternal_uniparental_isodisomy_count, 'matches_maternal_uniparental_isodisomy_count', ))
master_df['sig_prop_matches_paternal_uniparental_ambiguous_count'] = master_df.apply(is_significant, axis=1, args=(mean_matches_paternal_uniparental_ambiguous_count, 'matches_paternal_uniparental_ambiguous_count', ))
master_df['sig_prop_matches_paternal_uniparental_isodisomy_count'] = master_df.apply(is_significant, axis=1, args=(mean_matches_paternal_uniparental_isodisomy_count, 'matches_paternal_uniparental_isodisomy_count', ))

# save raw data to file
print ('Saving Raw UPD Metrics file to disk.')
master_df.to_csv(f'{output}_raw_data.csv', sep='\t', index=False)


print ('Calculating statistically significiant UPD events.')
# adjust supplied p value by the number of tests we are going to do - one for each type of mendellian error and block
p_value =  p_value / master_df.shape[0] / 4 

# now restrict to statistically significiant parts of chromosomes and merge contiguous blocks together
mat_amb = master_df[(master_df['sig_prop_matches_maternal_uniparental_ambiguous_count'] < p_value)]
mat_iso = master_df[(master_df['sig_prop_matches_maternal_uniparental_isodisomy_count'] < p_value)]
pat_amb = master_df[(master_df['sig_prop_matches_paternal_uniparental_ambiguous_count'] < p_value)]
pat_iso = master_df[(master_df['sig_prop_matches_paternal_uniparental_isodisomy_count'] < p_value)]


mat_amb_blocks = merge_contiguous_blocks(mat_amb, block_size, 'sig_prop_matches_maternal_uniparental_ambiguous_count','prop_matches_maternal_uniparental_ambiguous_count')
mat_iso_blocks = merge_contiguous_blocks(mat_iso, block_size, 'sig_prop_matches_maternal_uniparental_isodisomy_count','prop_matches_maternal_uniparental_isodisomy_count')
pat_amb_blocks = merge_contiguous_blocks(pat_amb, block_size, 'sig_prop_matches_paternal_uniparental_ambiguous_count','prop_matches_paternal_uniparental_ambiguous_count')
pat_iso_blocks = merge_contiguous_blocks(pat_iso, block_size, 'sig_prop_matches_paternal_uniparental_isodisomy_count','prop_matches_paternal_uniparental_isodisomy_count')

# save calls to file

upd_calls_list = []

	
for row in mat_amb_blocks:

	upd_calls_list.append(row + ['maternal_uniparental_ambiguous'])

for row in mat_iso_blocks:

	upd_calls_list.append(row + ['maternal_uniparental_isodisomy'])

for row in pat_amb_blocks:

	upd_calls_list.append(row + ['paternal_uniparental_ambiguous'])

for row in pat_iso_blocks:

	upd_calls_list.append(row + ['paternal_uniparental_isodisomy'])


# create dataframe from call list
calls_df = pd.DataFrame(upd_calls_list, columns =['chromosome', 'start', 'end', 'mean_p_value', 'mean_proportion_me', 'upd_event_signature'])

# if empty then write empty df
if calls_df.shape[0] == 0:

	calls_df = pd.DataFrame(upd_calls_list, columns =['chromosome', 'start', 'end', 'mean_p_value', 'mean_proportion_me', 'upd_event_signature', 'filter'])
	calls_df.to_csv(f'{output}_UPD_calls.csv', sep='\t', index=False)

else:
	
	# apply a filter  and write df
	calls_df['filter'] = calls_df.apply(apply_filters, axis=1, args=(min_blocks, min_proportion, block_size,))
	calls_df.sort_values('chromosome').to_csv(f'{output}_UPD_calls.csv', sep='\t', index=False)